// T4.C - all routines and data for printing to disk a full calendar

/*********************************************************************************
License for SLC - Tibetan Calendar software for Sherab Ling

Copyright (c) 2009-2011 Edward Henning

Permission is hereby granted, free of charge, to any person  obtaining a copy of
this software and associated documentation files (the "Software"), to deal in the
Software without restriction, including without limitation the rights to use, 
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the 
Software, and to permit persons to whom the Software is furnished to do so, subject 
to the following conditions: 

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
************************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <conio.h>
#include "tc.h"
#include "tc.ext"
#include "bcd.h"
#include "bcd.ext"

long int m_m, t_t;

char sreg_str[30];
char lmchg[50];
char byungphro_str[40];
char solar_term_str[60];
int full_print = 0; // Set this to 1 for printing all three files needed by Sherab Ling

void set_lang ( void );

// Routine to check for special days, festivals, anniversaries, etc.
// Many are currently commented out. Uncomment as needed.
int chk_spec ( long int m, long int t )
  {
    switch ( m )
      {
        case 1:
          if ( t == 1L )
            {
              sprintf ( outbuf, "%s",
                "From 1st to 15th, Demonstration of Miracles." );
              return (1);
            }
//        else if ( t == 14L )
//          {
//            sprintf ( outbuf, "%s",
//              "Anniversary of Milarepa." );
//            return (1);
//          }
//        else if ( t == 15L )
//          {
//            sprintf ( outbuf, "%s",
//              "Anniversary of Marpa." );
//            return (1);
//          }
        break;
//      case 2:
//        if ( t == 26L )
//          {
//            sprintf ( outbuf, "%s",
//              "Anniversary of Taranatha." );
//            return (1);
//          }
//      break;
        case 3:
//        if ( t == 3L )
//          {
//            sprintf ( outbuf, "%s", "Anniversary of Karma Pakshi." );
//            return (1);
//          }
          if ( t == 15L )
            {
              sprintf ( outbuf, "%s", "Revelation of the Kalacakra Tantra." );
              return (1);
            }
        break;
        case 4:
          // 7 for Phugpa, 8 for Tsurphu
          if ( t == 7L && tsurlug == 0 )
            {
              sprintf ( outbuf, "%s", "Birth of the Buddha." );
              return (1);
            }
          if ( t == 8L && tsurlug == 1 )
            {
              sprintf ( outbuf, "%s", "Birth of the Buddha." );
              return (1);
            }
          else if ( t == 15L )
            {
              sprintf ( outbuf, "%s",
                "Enlightenment and Parinirvana of the Buddha." );
              return (1);
            }
//       else if ( t == 29L )
//          {
//            sprintf ( outbuf, "%s",
//              "Anniversary of Mipham." );
//            return (1);
//          }
        break;
//      case 5:
//        if ( t == 10L )
//          {
//            sprintf ( outbuf, "%s", "Birth of Guru Rinpoche." );
//            return (1);
//          }
//        if ( t == 15L )
//          {
//            sprintf ( outbuf, "%s", "World Local Deities Day." );
//            return (1);
//          }
//      break;
        case 6:
          if ( t == 4L )
            {
              sprintf ( outbuf, "%s", "Turning of the Wheel of the Dharma." );
              return (1);
            }
          else if ( t == 15L )
            {
              sprintf ( outbuf, "%s",
                        "The Buddha's entry into the womb of his mother." );
              return (1);
            }
        break;
//      case 8:
//        if ( t == 19L )
//          {
//            sprintf ( outbuf, "%s",
//              "Anniversary of Dilgo Khyentse Rinpoche." );
//            return (1);
//          }
//        else if ( t == 23L )
//          {
//            sprintf ( outbuf, "%s",
//              "Anniversary of Karmapa Mikyo Dorje." );
//            return (1);
//          }
//      break;
        case 9:
//        if ( t == 16L )
//          {
//            sprintf ( outbuf, "%s",
//              "Anniversary of 16th Karmapa." );
//            return (1);
//          }
          if ( t == 22L )
            {
              sprintf ( outbuf, "%s",
                "Descent of the Buddha from the realm of the gods." );
              return (1);
            }
        break;
//      case 10:
//        if ( t == 10L )
//          {
//            sprintf ( outbuf, "%s",
//              "White Tara day." );
//            return (1);
//          }
//        else if ( t == 25L )
//          {
//            sprintf ( outbuf, "%s",
//              "Anniversary of Tshongkhapa." );
//            return (1);
//          }
//      break;
//      case 11:
//        if ( t == 3L )
//          {
//            sprintf ( outbuf, "%s",
//              "Anniversary of First Karmapa." );
//            return (1);
//          }
//        else if ( t == 6L )
//          {
//            sprintf ( outbuf, "%s",
//              "Nine bad omens." );
//            return (1);
//          }
//        else if ( t == 7L )
//          {
//            sprintf ( outbuf, "%s",
//              "Ten auspicious omens." );
//            return (1);
//          }
//        else if ( t == 26L )
//          {
//            sprintf ( outbuf, "%s",
//              "Anniversary of Jamgon Kongtrul." );
//            return (1);
//          }
//        break;
//      case 12:
//        if ( t == 18L )
//          {
//            sprintf ( outbuf, "%s",
//              "Anniversary of Longchen Rabjampa." );
//            return (1);
//          }
//        else if ( t == 29L )
//          {
//            sprintf ( outbuf, "%s",
//              "Mahakala Rituals." );
//            return (1);
//          }
//      break;
        default:
          ;
        break;
      }
    return (0);
  } // END - chk_spec ()

void set_lang ( void )
  {
    int i;
/***** SANSKRIT *********
    for ( i = 0; i < 8; ++i )
      cycpar[i] = cycparE[i];
    for ( i = 0; i < 27; ++i )
      lunmanP[i] = lunmanS[i];
    for ( i = 0; i < 27; ++i )
      yogaP[i] = yogaS[i];
    for ( i = 0; i < 11; ++i )
      byedP[i] = byedS[i];
*******************/

/***** TIBETAN **********/
    for ( i = 0; i < 27; ++i )
      lunmanP[i] = lunmanT[i];
    for ( i = 0; i < 27; ++i )
      yogaP[i] = yogaT[i];
    for ( i = 0; i < 11; ++i )
      byedP[i] = byedT[i];
/**************/
 } // END - set_lang

void get_phrochen ( long int *, long int *, long int, long int, long int, long int );

void get_phrochen ( long int *pc1, long int *pc2, long int doweek, long int lunmanx,
long int nxt_lm, long int monlong1 ) // VKP, 1,148
  {
    long int x;
    x = lunmanx;
    if ( x > 21L )
      x = x + 1L;
    else if ( x == 21L && monlong1 >= 30L )
      x = x + 1L;

    if ( doweek == 1L )
      *pc1 = x;
    else if ( doweek == 0L )
      *pc1 = x - 24L;
    else
      *pc1 = x - ( doweek - 1L ) * 4L;
    if ( *pc1 < 0L )
      *pc1 = *pc1 + 28L;

    x = nxt_lm;
    if ( x > 21L )
      x = x + 1L;
    else if ( x == 21L )
      x = x + 1L;

    if ( doweek == 1L )
      *pc2 = x;
    else if ( doweek == 0L )
      *pc2 = x - 24L;
    else
      *pc2 = x - ( doweek - 1L ) * 4L;
    if ( *pc2 < 0L )
      *pc2 = *pc2 + 28L;
  }

void zatse_phro ( long int *zapro, long int *tsepro, long int gza, long int tshe, long
int lm , long int monlong1 ) // This is for the five gzaphrod[]
  { // Not finished - still needs work
    *zapro = 0; *tsepro = 0;
    switch ( gza )
      {
        case 0L:
          if ( lm == 3L ) *zapro = 1L;
          else if ( lm == 12L ) *zapro = 2L;
          else if ( lm == 19L ) *zapro = 3L;
          if ( tshe == 6L || tshe == 7L || tshe == 21L || tshe == 22L )
            *tsepro = 1L;
          break;
        case 1L:
          if ( lm == 12L ) *zapro = 1L;
          else if ( lm == 16L ) *zapro = 2L;
          else if ( lm == 9L ) *zapro = 3L;
          if ( tshe == 11L || tshe == 12L || tshe == 26L || tshe == 27L )
            *tsepro = 1L;
          break;
        case 2L:
          if ( lm == 21L && monlong1 < 30L ) *zapro = 1L;  // first half only!
          else if ( lm == 2L ) *zapro = 2L;
          else if ( lm == 15L ) *zapro = 3L;
          if ( tshe == 10L || tshe == 11L || tshe == 25L || tshe == 26L )
            *tsepro = 1L;
          break;
        case 3L:
          if ( lm == 0L ) *zapro = 1L;
          else if ( lm == 23L ) *zapro = 2L;
          else if ( lm == 5L ) *zapro = 3L;
          if ( tshe == 9L || tshe == 10L || tshe == 24L || tshe == 25L )
            *tsepro = 1L;
          break;
        case 4L:
          if ( lm == 16L ) *zapro = 1L;
          else if ( lm == 0L ) *zapro = 2L;
          else if ( lm == 18L ) *zapro = 3L;
          if ( tshe == 2L || tshe == 3L || tshe == 17L || tshe == 18L )
            *tsepro = 1L;
          break;
        case 5L:
          if ( lm == 7L ) *zapro = 1L;
          else if ( lm == 4L ) *zapro = 2L;
          else if ( lm == 22L ) *zapro = 3L;
          if ( tshe == 5L || tshe == 6L || tshe == 20L || tshe == 21L )
            *tsepro = 1L;
          break;
        case 6L:
          if ( lm == 26L ) *zapro = 1L;
          else if ( lm == 3L ) *zapro = 4L;
          if ( tshe == 1L || tshe == 2L || tshe == 16L || tshe == 17L )
            *tsepro = 1L;
          break;
      }
  } // END - zatse_phro

void chk_tsephro ( char *str, long int phro1, long int phro2, long int gza1, long int gza2 )
  { // sreg_str, tsephro1, tsephro2, gzadag[1], gzadag[2]
    long int ts_chga, ts_chgb;
    if ( phro1 == 0L && phro2 == 0L )
      {
        *str = '\0';
        return;
      }

// First get time of lunar day change:

   ts_chgb = gza1;
   if ( gza2 > 30L )
     ++gza1;
   if ( gza1 > 60L )
     gza1 = 60L;
   ts_chga = ts_chgb / 5L;
   ts_chgb = ts_chgb - ts_chga * 5L + 1L;

    if ( phro1 == 1L && phro2 == 0L )
      {
        sprintf ( sreg_str, "sreg tshes %s gi %ld bar", cycanimt[ts_chga], ts_chgb );
      }
    else if ( phro1 == 0L && phro2 == 1L )
      {
        sprintf ( sreg_str, "%s gi %ld nas sreg tshes", cycanimt[ts_chga], ts_chgb );
      }
    else if ( phro1 == 1L && phro2 == 1L )
      {
        sprintf ( sreg_str, "sreg tshes" );
      }
  } // END - chk_tsephro

void byung_phro ( char *b_str, long int doweek, long int lm, long int monlong1 )
  { // I really need another source for these!!
    *b_str = '\0';
    switch ( doweek ) // This is unfinished, and wrong - there are only ten!!! ???
      {
        case 0L:
          if ( lm == 19L ) strcat ( b_str, byungphrod[0] );
          if ( lm == 14L ) strcat ( b_str, byungphrod[1] );
          if ( lm == 15L ) strcat ( b_str, byungphrod[2] );
          if ( lm == 19L ) strcat ( b_str, byungphrod[3] );
          if ( lm == 24L ) strcat ( b_str, byungphrod[4] );
          if ( lm == 12L ) strcat ( b_str, byungphrod[5] );
          if ( lm == 3L ) strcat ( b_str, byungphrod[7] );
          if ( lm == 1L ) strcat ( b_str, byungphrod[8] );
          if ( lm == 23L ) strcat ( b_str, byungphrod[9] );
          if ( lm == 10L ) strcat ( b_str, byungphrod[10] );
          break;
        case 1L:
          if ( lm == 9L ) strcat ( b_str, byungphrod[0] );
          if ( lm == 18L ) strcat ( b_str, byungphrod[1] );
          if ( lm == 18L ) strcat ( b_str, byungphrod[2] );
          if ( lm == 9L ) strcat ( b_str, byungphrod[3] );
          if ( lm == 3L ) strcat ( b_str, byungphrod[4] );
          if ( lm == 4L ) strcat ( b_str, byungphrod[5] );
          if ( lm == 25L ) strcat ( b_str, byungphrod[7] );
          if ( lm == 15L ) strcat ( b_str, byungphrod[8] );
          if ( lm == 2L ) strcat ( b_str, byungphrod[9] );
          if ( lm == 22L ) strcat ( b_str, byungphrod[10] );
          break;
        case 2L:
          if ( lm == 15L ) strcat ( b_str, byungphrod[0] );
          if ( lm == 21L && monlong1 >= 30L )
            strcat ( b_str, byungphrod[1] ); //second half only
          if ( lm == 22L ) strcat ( b_str, byungphrod[2] );
          if ( lm == 15L ) strcat ( b_str, byungphrod[3] );
          if ( lm == 14L ) strcat ( b_str, byungphrod[4] );
          if ( lm == 17L ) strcat ( b_str, byungphrod[5] );
          if ( lm == 8L ) strcat ( b_str, byungphrod[7] );
          if ( lm == 17L ) strcat ( b_str, byungphrod[8] );
          if ( lm == 11L ) strcat ( b_str, byungphrod[9] );
          if ( lm == 15L ) strcat ( b_str, byungphrod[10] );
          break;
        case 3L:
          if ( lm == 5L ) strcat ( b_str, byungphrod[0] );
          if ( lm == 25L ) strcat ( b_str, byungphrod[1] );
          if ( lm == 25L ) strcat ( b_str, byungphrod[2] );
          if ( lm == 5L ) strcat ( b_str, byungphrod[3] );
          if ( lm == 7L ) strcat ( b_str, byungphrod[4] );
          if ( lm == 2L ) strcat ( b_str, byungphrod[5] );
          if ( lm == 14L ) strcat ( b_str, byungphrod[7] );
          if ( lm == 26L ) strcat ( b_str, byungphrod[8] );
          if ( lm == 8L ) strcat ( b_str, byungphrod[9] );
          if ( lm == 16L ) strcat ( b_str, byungphrod[10] );
          break;
        case 4L:
          if ( lm == 18L ) strcat ( b_str, byungphrod[0] );
          if ( lm == 2L ) strcat ( b_str, byungphrod[1] );
          if ( lm == 2L ) strcat ( b_str, byungphrod[2] );
          if ( lm == 18L ) strcat ( b_str, byungphrod[3] );
          if ( lm == 16L ) strcat ( b_str, byungphrod[4] );
          if ( lm == 19L ) strcat ( b_str, byungphrod[5] );
          if ( lm == 17L ) strcat ( b_str, byungphrod[7] );
          if ( lm == 12L ) strcat ( b_str, byungphrod[8] );
          if ( lm == 19L ) strcat ( b_str, byungphrod[9] );
          if ( lm == 17L ) strcat ( b_str, byungphrod[10] );
          break;
        case 5L:
          if ( lm == 11L ) strcat ( b_str, byungphrod[0] );
          if ( lm == 6L ) strcat ( b_str, byungphrod[1] );
          if ( lm == 7L ) strcat ( b_str, byungphrod[2] );
          if ( lm == 24L ) strcat ( b_str, byungphrod[3] );
          if ( lm == 25L ) strcat ( b_str, byungphrod[4] );
          if ( lm == 23L ) strcat ( b_str, byungphrod[5] );
          if ( lm == 11L ) strcat ( b_str, byungphrod[7] );
          if ( lm == 3L ) strcat ( b_str, byungphrod[8] );
          if ( lm == 4L ) strcat ( b_str, byungphrod[9] );
          if ( lm == 21L && monlong1 < 30L )
            strcat ( b_str, byungphrod[10] );
          break;
        case 6L:
          if ( lm == 3L ) strcat ( b_str, byungphrod[0] );
          if ( lm == 10L ) strcat ( b_str, byungphrod[1] );
          if ( lm == 10L ) strcat ( b_str, byungphrod[2] );
          if ( lm == 3L ) strcat ( b_str, byungphrod[3] );
          if ( lm == 4L ) strcat ( b_str, byungphrod[4] );
          if ( lm == 0L ) strcat ( b_str, byungphrod[5] );
          if ( lm == 2L ) strcat ( b_str, byungphrod[7] );
          if ( lm == 6L ) strcat ( b_str, byungphrod[8] );
          if ( lm == 7L ) strcat ( b_str, byungphrod[9] );
          if ( lm == 3L ) strcat ( b_str, byungphrod[10] );
          break;
      }
  } // byung_phro

long int get_chimonth ( long int gm, long int mnum )
  {
    long int M;
    if ( gm >= 14833L && gm <= 14847L )
      M = mnum + 1L;
    else if ( gm >= 14866L && gm <= 14882L )
      M = mnum + 1L;
    else if ( gm >= 14900L && gm <= 14918L )
      M = mnum + 1L;
    else if ( gm >= 14933L && gm <= 14948L )
      M = mnum + 1L;
    else
      M = mnum;
    if ( M > 12L )
      M = 12L;
    return ( M );
  } // END - get_chimonth


int chk_solar_term ( long int *gza, long int *nyib, char *t_str )
//         chk_solar_term ( gzadag, nyibar, solar_term_str );
  {
    long int sundb1, sundb2; // Mean sun at daybreak, begin and end
    long int sunmid;
    long int x; //, y;
    long int s1, r, s2, s2t, s2t1, s2t2, s2t3;
    long int s2long, s2long1, s2long2, s2long3;
    long int f; // fraction of day at lunar day. out of 21600
// To nearest dbugs, sun moves 0;4,26,2 -  1598/21600

    f = ( gzadag[1] * 60L + gzadag[2] ) * 6L + gzadag[3];
    sundb1 = (( nyib[0] * 60L + nyib[1] ) * 60L + nyib[2] ) * 6L + nyib[3];
    sunmid = sundb1;
    sundb1 = sundb1 - ( f * 1598L ) / 21600L;
    sundb2 = sundb1 + 1598L;

// Now, find where we are in solar terms. First is actually 3 sgang:

    if ( sundb1 < 11340L )
      sundb1 = sundb1 + 583200L;
    x = sundb1 - 11340L;
    s1 = x / 24300L;
    r = x - ( s1 * 24300L );

//  printf ( "Term index = %ld, remainder = %ld\n", s1, r );

// If index is zero and remainder positive, we have passed "3 sgang"
// It appears that the remainder is never zero.

    if ( sundb2 < 11340L )
      sundb2 = sundb2 + 583200L;
    x = sundb2 - 11340L;
    s2 = x / 24300L;
    r = x - ( s2 * 24300L );

//  printf ( "Second term index = %ld, remainder = %ld\n", s2, r );

// Convert to solar terms, "1 dbugs" = 1, etc.

    s1 = s1 + 6L;
    if ( s1 > 24L )
      s1 = s1 - 24L;
    s2 = s2 + 6L;
    if ( s2 > 24L )
      s2 = s2 - 24L;

    if ( s1 == s2 )
      return (0); // No solar term

// Now, find the time of the term.
// between each solar term, 24300 "dbugs"
// Solar daily motion = 1598 dbugs

    s2long = ( s2 - 6L ) * 24300L + 11340;
    if ( s2long < 0L )
      s2long = s2long + 583200L;
//  printf ( "Step 1, S2long = %ld\n", s2long );

    if ( s2long < 0L )
      s2long = s2long + 583200L;
    else if ( s2 == 1L )
      s2long = s2long + 583200L;

    if ( sundb1 > s2long ) // It must have been incremented, so do the same:
      s2long = s2long + 583200L;

    s2t = ( 21600L * ( s2long - sundb1 ))/ 1598L;
    s2t2 = s2t / 6L;
    s2t3 = s2t - s2t2 * 6L;
    s2t1 = s2t2 / 60L;
    s2t2 = s2t2 - s2t1 * 60L;

// solar_term_str[60];

    if ( s2long < 0L )
      s2long = s2long + 583200L;
    s2long = ( s2long + 3L ) / 6L;
    s2long2 = s2long / 60L;
    s2long3 = s2long % 60L;
    s2long1 = s2long2 / 60L;
    s2long2 = s2long2 % 60L;

    x = ( 1L + s2 ) / 2L;

    if ( s2 % 2L ) // If index odd, then "dbugs"
      {
        sprintf ( solar_term_str, "%ld,%ld,%ld %ld dbugs %ld;%ld,%ld",
                  s2t1, s2t2, s2t3, x, s2long1, s2long2, s2long3 );
      }
    else // "sgang"
      {
        sprintf ( solar_term_str, "%ld,%ld,%ld %ld sgang %ld;%ld,%ld",
                  s2t1, s2t2, s2t3, x, s2long1, s2long2, s2long3 );
      }
    return (1); // solar term found
  } // chk_solar_term

void mak_lmchange_string ( long int *f_ml, long int *s_ml, long int dow )
  {
    int i;
    long int fst_ml[6], scd_ml[6];
    long int lm1, lm2, lmx;
    long int chga, chgb, nxtlm;

    if ( f_ml[0] == s_ml[0] )     
      {
        lmchg[0] = '\0';
        return;
      }

    for ( i = 0; i < 6; ++i )
      {
        fst_ml[i] = f_ml[i];
        scd_ml[i] = s_ml[i];
      }
      
    nxtlm = scd_ml[0];      
    if ( scd_ml[0] == 0L )
      scd_ml[0] = 27L;
      
     scd_ml[0] = scd_ml[0] - fst_ml[0];
     fst_ml[0] = 0L;

// reduce to "dbugs":

     lm1 = ( fst_ml[1] * 60L + fst_ml[2] ) * 6L + fst_ml[3];
     lm2 = ( ( scd_ml[0] * 60L + scd_ml[1] ) * 60L + scd_ml[2] ) * 6L + scd_ml[3];
     lmx = s_ml[0];
     
// Time to lunar mansion change:     
     
     chgb = ( 60L * ( 60L * 60L * 6L - lm1 ) ) / ( lm2 - lm1 );
     chga = chgb / 5L;
     chgb = chgb - chga * 5L + 1L;
     sprintf ( lmchg, "%s gi %ld nas %s(%ld). %s-%s", cycanimt[chga], chgb, lunmanP[nxtlm], nxtlm, 
               fourelem[ dowkelem[ (int) doweek ]], fourelem[ lunmanelem[ lmx ]] );
   } // END - mak_lmchange_string

long int calc_byedpa ( long int * mlong, long int * ndag )
  {
    long int byedpa, x;
    clear_a_b ();
    sub_gen ( lista, mlong, ndag, 27L, sun_f ); // AT SUNRISE FOR MOON
    byedpa = lista[0] * 60 + (int) lista[1];
    byedpa = byedpa / 27L;     
    if ( byedpa == 0L )
      x = 7L;
    else if ( byedpa == 57L )
      x = 8L;
    else if ( byedpa == 58L )
      x = 9L;
    else if ( byedpa == 59L )
      x = 10L;
    else
      x = ( byedpa - 1L ) % 7L;      
    return (x);      
  } // END - calc_byedpa

void prn_bir2_cal (void) 
  {
    int i, more, n, newmth, lhag, chad, nextchad, nextlhag;
    long int zaphro1, tsephro1; // for daily information
    long int zaphro2, tsephro2; // for daily information
    long int lmx[6], lm1[6], lm2[6];
    long int gzafrac1, gzafrac2;
    int intercal, delaymth;
    int yr_gender;    // 1 = female, odd; 0 = male, even
    int mt_gender;    // 1 = female, odd; 0 = male, even
    long int mthnum; // Adjusted, without negative.
    long int gen_mth; // General monthnumber count from Kalacakra epoch
    long int cur_year, last_year, x, y;
    long int ltt, zla0, zla1;
    long int lastjd = 0L;
    long int curjd, nextjd; // Last and next JD calculations.
    long int lunmanx, sbyorba, /*byedpa,*/ byedpax;
    long int /*prv_lunmanx,*/ prv_sbyorba, /*prv_byedpa,*/ prv_byedpax;
    long int fst_lunmanx;
    long int bir_spizag;
    long int Cmonth;
    long int sd_clunmanx;
    long int ld_parx; //, sd_parx;
    long int ld_smex, sd_smex;
    long int yr_animx, mt_animx, ld_animx, sd_animx;
    long int yr_elemx, mt_elemx, /*ld_elemx,*/ sd_elemx;
    long int /*lm_chga, lm_chgb,*/ nxt_lm;
    long int ZS_s, ZS_d, ZS_m;  // Zodiac signs, minutes, seconds;
    long int phrochen1, phrochen2;
//  long int day1sun, day2sun; // Data for solar longitude, in dbugs
//  int e, g;
    FILE *fptgt1, *fptgt2;

    set_lang ();
    gza_short_flg = 1; // Flag for less accurate weekday adjustment
    cls ();
//  Changed for Win32 console app:        
//  pos (0,0);
    printf ( "%s\n", e_str[epch] );
    printf ("Enter the starting year: ");
    scanf ("%ld", &ty);
    printf ("\n");

    cur_year = ty;

    printf ("Enter the finish year: ");
    scanf ("%ld", &last_year );
    printf ("\n");

    birdonextyear:

    ty = cur_year;
    tm = 1L;
    zeromthfg = 0;

    sprintf ( filenam, "sla_%ld.txt", ty );
    fptgt1 = fopen ( filenam, "w" );
    if ( fptgt1 == NULL )
      {
        printf ( "Error opening target file!\n" );
        getch ();
      }

    if ( full_print )                                  
      {
        sprintf ( filenam, "sl2_%ld.txt", ty );
        fptgt = fopen ( filenam, "w" );
        if ( fptgt == NULL )
          {
            printf ( "Error opening target file!\n" );
            getch ();
          }

        sprintf ( filenam, "slb_%ld.txt", ty );
        fptgt2 = fopen ( filenam, "w" );
        if ( fptgt == NULL )
          {
            printf ( "Error opening target file!\n" );
            getch ();
          }
     }

// tt not yet set
// First, check to see if we are not at adjusted 12th month:

    zla_dag (ty, tm); // But only if moving one month at a time!

    zla0 = zladag[0];
    zla1 = zladag[1];

    adj_zla ();

//  printf ( "\nStart of year loop:\n" );
//  printf ("The ZLA-DAG is: %ld, %ld\n", zladag[0], zladag[1]);
//  printf ("Month (tm) = %ld\n", tm );
//  printf ("Adjusted = %ld\n\n", adj_mth );

    if ( adj_mth == 12L )
      {
//      printf ( "Starting with 12th month.\n" );
//      getch ();
        if ( !zeromthfg )       // Check this out again...
          ++tm;
      }
    else if ( adj_mth == -1L )
      {
        zladag[0] = zla0;
        zladag[1] = zla1;
        zeromthfg = 0;
      }
    n = 0;
    more = 1;
    newmth = 1;
    do   // START OF MONTHLY LOOP *******************************
      {
        intercal = 0;
        if ( adj_mth == -1L )
          intercal = 1;
        delaymth = 0;
        if ( newmth )
          {
//          printf ( "\nStart of month loop:\n" );
            if ( !zeromthfg )   // We need to use the same data, twice.
              zla_dag (ty, tm); // But only if moving one month at a time!
            adj_zla ();
            cur_mth = zladag[0];  // NEEDED FOR GENERAL DAY
            gen_mth = cur_mth + 14608L; // General month number
            gza_dru (cur_mth);
            nyi_dru (cur_mth);
            rilchaf ( cur_mth ); 
// DO WE HAVE NEW YEAR?

            if ( adj_mth == -1 || adj_mth == 1 )
              {
                if ( ty < 7L )
                  {
                    yr_animx = ( ty + 1200L - 7L ) % 12L;
                    yr_elemx = ( ( ty + 1200L ) / 2L - 3L ) % 5L;
                    yr_gender = (int) ( ty + 1200L ) % 2; // 1 = female, odd; 0
                  }
                else
                  {
                    yr_animx = ( ty - 7L ) % 12L;
                    yr_elemx = ( ty / 2L - 3L ) % 5L;
                    yr_gender = (int) ty % 2; // 1 = female, odd; 0 = male
                  }
                sprintf ( outbuf, "\nNew Year: %ld, %s-%s-%s\n", ty,
                                  cycelem1[ (int) yr_elemx ],
                                  cycgendE[ yr_gender ],
                                  cycanim[ (int) yr_animx ] );
                if ( full_print )                                  
                  fprintf ( fptgt, "%s", outbuf );
                fprintf ( fptgt1, "%s", outbuf );
              }
          }

        if ( adj_mth < 0L )
          {
            intercal = 1;
            delaymth = 0;
          }
        else if ( adj_mth > 0L && ( zladag[1] == zlasho1+2L || zladag[1] == zlasho2+2L ) )
          {
            intercal = 0;
            delaymth = 1;
          }

        if ( adj_mth < 0L )
          mthnum = -adj_mth;
        else
          mthnum = adj_mth;

        mt_animx = mthnum % 12L;
        Cmonth = mthnum + 2L;
        if ( Cmonth > 12L )
          Cmonth -= 12L;

//      printf ("Month animal ref: %ld\n", mt_animx ); // Rabbit = 0
//      getch ();

        mt_elemx = yr_elemx + 1L + ( mthnum + 1L ) / 2L;

        if ( yr_gender == 0 ) // Male year, even CE number
          {
            if ( mthnum == 11L || mthnum == 12L )
              mt_elemx = yr_elemx + ( (int) mthnum + 1L ) / 2;
          }

        mt_elemx = mt_elemx % 5L;
        mt_gender = ( mthnum + 1 ) % 2; // There is no month zero

        if ( intercal )
          sprintf ( outbuf,
          "Tibetan Lunar Month: %ld (Intercalary) - %s-%s-%s",
          -adj_mth, cycelem1[ (int) mt_elemx ],
          cycgendE[ mt_gender ], cycanim[ (int) mt_animx ] );

        else if ( delaymth )
          sprintf ( outbuf, "Tibetan Lunar Month: %ld (Delayed) - %s-%s-%s",
          adj_mth, cycelem1[ (int) mt_elemx ],
          cycgendE[ mt_gender ], cycanim[ (int) mt_animx ] );

        else
          sprintf ( outbuf, "Tibetan Lunar Month: %ld - %s-%s-%s: %ld",
          adj_mth, cycelem1[ (int) mt_elemx ],
          cycgendE[ mt_gender ], cycanim[ (int) mt_animx ], gen_mth );

        printf ( "%s\n", outbuf );
        if ( full_print )                                  
          fprintf ( fptgt, "\n%s\n", outbuf );
        fprintf ( fptgt1, "\n%s\n", outbuf );

        sprintf ( outbuf, "Month: %ld;%ld", zladag[0], zladag[1] );
        if ( full_print )                                         
          fprintf ( fptgt, "\n%s\n", outbuf );
        fprintf ( fptgt1, "\n%s\n", outbuf );
        sprintf ( outbuf, "Anomaly: %ld;%ld", rilcha[0], rilcha[1] );
        if ( full_print )                                  
          fprintf ( fptgt, "%s\n", outbuf );
        fprintf ( fptgt1, "%s\n", outbuf );

        tse_dru (0L);
        nyi_lon (0L);
        add_gen (tsebar, gzadru, tsedru, 7L, gza_f);
        add_gen (nyibar, nyidru, nyilon, 27L, sun_f );
        nyi_dag (nyibar); // MUST BE DONE FIRST
        gza_dag (tsebar);
        tt = 0L;
        spi_zagf ();

        sprintf ( outbuf, "Mean Weekday: %ld;%ld,%ld,%ld,%ld",
                          gzadru[0], gzadru[1], gzadru[2], gzadru[3], gzadru[4] );
        if ( full_print )                                                            
          fprintf ( fptgt, "%s\n", outbuf );
        fprintf ( fptgt1, "%s\n", outbuf );
        x = nyidru[4] / 6811L;
        y = nyidru[4] % 6811L;
        sprintf ( outbuf, "Mean Sun: %ld;%ld,%ld,%ld,%ld,%ld",
                          nyidru[0], nyidru[1], nyidru[2], nyidru[3], x, y );
        if ( full_print )                                  
          fprintf ( fptgt, "%s\n", outbuf );
        fprintf ( fptgt1, "%s\n", outbuf );

// Now, calculate planetary data and Rahu: 

// FIRST - RAHU:

         do_rahu ( cur_mth, tt );
         sprintf ( outbuf,
         "Rahu: 15th: %ld;%ld,%ld,%ld,%ld - \
30th: %ld;%ld,%ld,%ld,%ld", rahudong[0], rahudong[1], rahudong[2], rahudong[3],
          rahudong[4], rahudong30[0], rahudong30[1], rahudong30[2], rahudong30[3],
          rahudong30[4] );
         if ( full_print )                                  
           fprintf ( fptgt, "\n%s\n\n", outbuf );
         fprintf ( fptgt1, "\n%s\n\n", outbuf );

// NOW - PLANETS:

        tt = 1L;
        ltt = 1L;
        Cmonth = get_chimonth ( gen_mth, mthnum );
        
// START OF DAY LOOP - START OF DAY LOOP - START OF DAY LOOP
        
        do
          {   // START OF DAY LOOP *******************************
            nextlhag = 0;
            lhag = 0;
            chad = 0;
            nextchad = 0;

// Work out previous and next lunar days to determine CHAD or LHAG and prepare other data
            
            tt = ltt - 1L;
            tse_dru (tt);
            nyi_lon (tt);
            add_gen (tsebar, gzadru, tsedru, 7L, gza_f);
            add_gen (nyibar, nyidru, nyilon, 27L, sun_f );
            nyi_dag (nyibar); // MUST BE DONE BEFORE gza_dag
            gza_dag (tsebar);
            spi_zagf ();
            lastjd = juldat;
            for ( i = 0; i < 5; ++i )
              monlong[i] = 0L;
            monlong[1] = 54L;  // 1/30th of a revolution
            mul_gen ( monlong, monlong, tt, 27L, sun_f );
            add_gen ( monlong, monlong, nyidag, 27L, sun_f );
            for ( i = 0; i < 6; ++i )
              {
                prv_nyidag[i] = nyidag[i];
                prv_gzadag[i] = gzadag[i];
                prv_monlong[i] = monlong[i];
              }

// PROBABLY DON'T NEED THIS BLOCK - O YES WE DO!

            tt = ltt + 1L;
            tse_dru (tt);
            nyi_lon (tt);
            add_gen (tsebar, gzadru, tsedru, 7L, gza_f);
            add_gen (nyibar, nyidru, nyilon, 27L, sun_f );
            nyi_dag (nyibar); // MUST BE DONE FIRST
            gza_dag (tsebar);
            spi_zagf ();
            nextjd = juldat;
            for ( i = 0; i < 5; ++i )
              monlong[i] = 0L;
            monlong[1] = 54L;  // 1/30th of a revolution
            mul_gen ( monlong, monlong, tt, 27L, sun_f );
            add_gen ( monlong, monlong, nyidag, 27L, sun_f );
            for ( i = 0; i < 6; ++i )
              {
                nxt_nyidag[i] = nyidag[i];
                nxt_gzadag[i] = gzadag[i];
                nxt_monlong[i] = monlong[i];
              }

// NOW, CALCULATE FOR CURRENT TITHI:

            tt = ltt;
            tse_dru (tt);
            nyi_lon (tt);
            add_gen (tsebar, gzadru, tsedru, 7L, gza_f);
            add_gen (nyibar, nyidru, nyilon, 27L, sun_f );
            nyi_dag (nyibar); // MUST BE DONE FIRST
            gza_dag (tsebar);
            for ( i = 0; i < 5; ++i )
              monlong[i] = 0L;
            monlong[1] = 54L;  // 1/30th of a revolution
            mul_gen ( monlong, monlong, tt, 27L, sun_f );
            add_gen ( monlong, monlong, nyidag, 27L, sun_f );
            spi_zagf ();
            curjd = juldat;
            if ( curjd == lastjd + 2L )
              lhag = 1;
            if ( nextjd == curjd + 2L )
              nextlhag = 1;
            if ( curjd == lastjd )
              chad = 1;
            if ( curjd == nextjd )
              nextchad = 1;

            if (lhag)
              bir_spizag = spizag - 1L;
            else
              bir_spizag = spizag;
            ++bir_spizag; // For their strange way of doing things.

// Solar day data:

            sd_animx = ( juldat - 2L ) % 12L;
            sd_elemx = ( ( juldat - 3L ) / 2L ) % 5L;
            sd_clunmanx = ( juldat - 17L ) % 28L;
            sd_smex = ( juldat - 2L ) % 9L + 1L;  // CHANGED 31/8/2004

// Lunar day data:

// Calculations lower down maybe wrong!!! Need to check??

// If Chinese month is number 1, Trigram is Li, index = 1
            ld_parx = ( Cmonth - 1L ) * 30L + tt + 4L;
//          ld_parx = ( Cmonth - 1L ) * 30L + tt;
            ld_parx = ld_parx % 8L;
// If Chinese month is number 1, "sme ba" is 1
            ld_smex = ( Cmonth - 1L ) * 30L + tt + 6L;
//          ld_smex = ( Cmonth - 1L ) * 30L + tt;
            ld_smex = ld_smex % 9L;
            if ( ld_smex == 0L )
              ld_smex = 9L;
// If Chinese month is number 1, Animal is Tiger, index = 11
//          ld_animx = 6L * ( ( mthnum - 1L ) % 2L ) + tt + 10L;

            ld_animx = ( Cmonth - 1L ) * 30L + tt + 10L;
            ld_animx = ld_animx % 12L;

            if ( chad )   // THIS IS FOR OMITTED LUNAR DAY
              {
// LINE 1 - Omitted day.
                sprintf ( outbuf, "%ld. Omitted: %s %s %ld",
                          tt,
                          cycanim[ (int) ld_animx ],
                          cycpar[ (int) ld_parx ], ld_smex );

// Should include "gza'" for end of lunar day, plus solar longitude.

                if ( full_print )                                  
                  fprintf ( fptgt, "%s\n\n", outbuf );
                fprintf ( fptgt1, "%s\n", outbuf );
              }
            else  // IF NORMAL OR DUPLICATED:
              {
// NYIDAG IS NOT CHANGED AFTER HERE, SO WE CAN CALCULATE ZODIACAL SIGN:
// Zodiac signs, minutes, seconds -  ZS_s, ZS_d, ZS_m;

                ZS_m = ( ( nyidag[0] * 60L + nyidag[1] ) * 60L + nyidag[2] ) *
                         6L + nyidag[3];
                l2bcd ( bcdx0, ZS_m );
                mulbcdl ( bcdx0, bcdx0, 21600L );
                divbcdl ( bcdx1, bcdx0, 583200L );
                ZS_m = bcd2l ( bcdx1 );

                ZS_d = ZS_m / 60L;
                ZS_m = ZS_m % 60L;
                ZS_s = ZS_d / 30L;
                ZS_d = ZS_d % 30L;
                if ( lhag )  // IF WE HAVE DUPLICATED LUNAR DATE - BOTH IN THIS BLOCK
                {
                // Put here any special calculations for extra day -- *NEW*
                // At this point, the gzadag, nyidag & monlong are correct for
                // the second of the two days with same tithi. We need for the
                // first.
                // Previous tithi ended just before start of day we need.
               
// prv_monlong is the moon longitude at beginning of current lunar day.

                clear_a_b ();                
                listb[0] = 1L;
                for ( i = 1; i < 5; ++i )
                  lista[i] = prv_gzadag[i];
                sub_gen ( listb, listb, lista, 7L, gza_f );

//              listb[4] = ( sun_f * listb[4] ) / gza_f;                
                l2bcd ( bcdx0, sun_f );
                mulbcdl ( bcdx0, bcdx0, listb[4] );
                divbcdl ( bcdx1, bcdx0, gza_f );
                listb[4] = bcd2l ( bcdx1 ); 
                add_gen ( fst_monlong, prv_monlong, listb, 27L, sun_f );
// This is now the moon longitude at daybreak of first duplicated day
                fst_lunmanx = fst_monlong[0]; 
                printf ( "Previous lunar day:" ); // XXX
                prn_lst ( prv_monlong );  // XXX
                printf ( "First solar day:   " );  // XXX
                prn_lst ( fst_monlong );  // XXX

                listb[0] = 0L;
                for ( i = 1; i < 5; ++i )
                  listb[i] = gzadag[i];
//              sub_gen ( listb, listb, lista, 7L, gza_f );

                l2bcd ( bcdx0, sun_f );
                mulbcdl ( bcdx0, bcdx0, listb[4] );
                divbcdl ( bcdx1, bcdx0, gza_f );
                listb[4] = bcd2l ( bcdx1 ); 
                sub_gen ( cur_monlong, monlong, listb, 27L, sun_f );
// This is now the moon longitude at daybreak of second duplicated day
                lunmanx = cur_monlong[0]; 

// Now, for end of second duplicated day.
// First, if lunar day is not omitted - it can happen -- see, eg., phugpa 6th of 9th month 1457

                if ( nxt_gzadag[0] != gzadag[0] )
                  {
                    listb[0] = 0L;
                    for ( i = 1; i < 5; ++i )
                      listb[i] = nxt_gzadag[i];
                    l2bcd ( bcdx0, sun_f );
                    mulbcdl ( bcdx0, bcdx0, listb[4] );
                    divbcdl ( bcdx1, bcdx0, gza_f );
                    listb[4] = bcd2l ( bcdx1 ); 
                    sub_gen ( scd_monlong, nxt_monlong, listb, 27L, sun_f );
// This is now the moon longitude at end of second duplicated day
                  }
                else // if following lunar day IS omitted
                  {  
                    clear_a_b ();                
                    listb[0] = 1L;
                    for ( i = 1; i < 5; ++i )
                      lista[i] = nxt_gzadag[i];
                    sub_gen ( listb, listb, lista, 7L, gza_f );

//              listb[4] = ( sun_f * listb[4] ) / gza_f;                
                
                    l2bcd ( bcdx0, sun_f );
                    mulbcdl ( bcdx0, bcdx0, listb[4] );
                    divbcdl ( bcdx1, bcdx0, gza_f );
                    listb[4] = bcd2l ( bcdx1 ); 
                    add_gen ( scd_monlong, nxt_monlong, listb, 27L, sun_f );
// This is now the moon longitude at end of second duplicated day
                  }

                mak_lmchange_string ( fst_monlong, cur_monlong, doweek );

// NOW CALCULATE SBYOR BA:

                add_gen ( lista, prv_monlong, prv_nyidag, 27L, sun_f );
                prv_sbyorba = lista[0];
                for ( i = 0; i < 6; ++i )
                  prv_sbyorlong[i] = lista[i];
                add_gen ( lista, cur_monlong, nyidag, 27L, sun_f );
                sbyorba = lista[0];
                for ( i = 0; i < 6; ++i )
                  prv_sbyorlong[i] = lista[i];                  

// NOW CALCULATE BYED PA:
                
                prv_byedpax = calc_byedpa ( fst_monlong, prv_nyidag );                
                byedpax = calc_byedpa ( cur_monlong, nyidag );                

                juldat = juldat - 1L;
                jul2date ( juldat ); // This is normally done by spi_zagf

// Solar day data:

                sd_animx = ( juldat - 2L ) % 12L;
                sd_elemx = ( ( juldat - 3L ) / 2L ) % 5L;
                sd_clunmanx = ( juldat - 17L ) % 28L;
                sd_smex = ( juldat - 2L ) % 9L + 1L;  // CHANGED 31/8/2004

// Lunar day data:

                ld_parx = ( Cmonth - 1L ) * 30L + tt + 4L;
                ld_parx = ld_parx % 8L;
                ld_smex = ( Cmonth - 1L ) * 30L + tt + 6L;
                ld_smex = ld_smex % 9L;
                if ( ld_smex == 0L )
                  ld_smex = 9L;
                ld_animx = 6L * ( ( Cmonth - 1L ) % 2L ) + tt + 10L;
                ld_animx = ld_animx % 12L;

// LINE 1. - FRED: For first of duplicated day. only
                sprintf ( outbuf, "%ld: %s %s %s. %s(%ld). %s-%s; %s %ld %s %ld",
                          tt, //                     XXX
                          rgyalsogs[(tt-1L)%5L], // NEW 17/4/2010
                          tseyang[(tt-1L)%5L], // NEW 17/4/2010
                          dayoweek[ (int) doweek ],
                          lunmanP[ fst_lunmanx ], fst_lunmanx, // XX
                          fourelem[ dowkelem[ (int) doweek ] ],
                          fourelem[ lunmanelem[ fst_lunmanx ] ],
                          lmchg,
                          wd, wmonths[ (int) (wm-1L) ], wy);
                  if ( full_print )                                  
                    fprintf ( fptgt, "%s\n", outbuf );
                  fprintf ( fptgt1, "%s\n", outbuf );

// LINE 2.
                sprintf ( outbuf, "%s, %s, %s, %s %ld",
                          yogaP[ prv_sbyorba ], 
                          byedP[ prv_byedpax ],
                          cycanim[ ld_animx ],
                          cycpar[ (int) ld_parx ], ld_smex );
                fprintf ( fptgt1, "\0x20\0x20%s\n", outbuf );

// LINE 3.
                sprintf ( outbuf,
//              "%ld;%ld,%ld %ld;%ld,%ld %ld;%ld,%ld %ld;%ld,%ld %ld;%ld,%ld",
   "%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld",
                       ( prv_gzadag[0] + 1L ) % 7L, 60L, 0L,
//                     prv_gzadag[0], prv_gzadag[1], prv_gzadag[2],
                       prv_monlong[0], prv_monlong[1], prv_monlong[2],
                       nyidag[0], nyidag[1], nyidag[2],
                       prv_sbyorlong[0], prv_sbyorlong[1], prv_sbyorlong[2],
                       ZS_s, ZS_d, ZS_m );
                  if ( full_print )                                                         
                    fprintf ( fptgt, "%s\n", outbuf );
                  if ( full_print )                                                      
                    fprintf ( fptgt2, "%s\n", outbuf );

// LINE 4.
                    sprintf ( outbuf, "Solar: %s-%s. %s %ld" ,
                              cycelem1[ (int) sd_elemx ],
                              cycanim[ (int) sd_animx ],
                              Clunman[ sd_clunmanx ],
                              sd_smex );
                if ( full_print )                                                                
                  fprintf ( fptgt, "\x20\x20%s\n", outbuf );
                fprintf ( fptgt1, "\x20\x20%s\n", outbuf );

// LINE 5.

                sprintf ( outbuf, "%ld %s %ld",
                          wd, wmonths[ (int) (wm-1L) ], wy);

                if ( full_print )                                  
                  fprintf ( fptgt, "\x20\x20%s\n", outbuf );
                fprintf ( fptgt1, "\x20\x20%s\n", outbuf );


// NOW DO SECOND OF REPEATED LUNAR DAYS

            juldat = juldat + 1L;  // THIS IS READJUSTMENT!
            jul2date ( juldat );

// Solar day data:   // This is a brute force method I developed.

            sd_animx = ( juldat - 2L ) % 12L;
            sd_elemx = ( ( juldat - 3L ) / 2L ) % 5L;
            sd_clunmanx = ( juldat - 17L ) % 28L;
            sd_smex = ( juldat - 2L ) % 9L + 1L;  // CHANGED 31/8/2004

// PUT PLANETS IN HERE:

            do_plans ( bir_spizag ); // In T3.C
            sprintf ( outbuf, "Planets (%ld):", bir_spizag );
            if ( full_print )                                  
              fprintf ( fptgt, "\n%s\n\n", outbuf );

      sprintf ( outbuf, "Mars (%ld),    dal-dag: %ld;%ld,%ld,%ld,%ld\
\tmyur-dag: %ld;%ld,%ld,%ld,%ld", mar_gz, mardaldag[0], mardaldag[1], mardaldag[2],
  mardaldag[3], mardaldag[4], marmurdag[0], marmurdag[1], marmurdag[2], marmurdag[3],
   marmurdag[4] );
           if ( full_print )                                  
             fprintf ( fptgt, "%s\n", outbuf );

      sprintf ( outbuf, "Jupiter (%ld), dal-dag: %ld;%ld,%ld,%ld,%ld\
\tmyur-dag: %ld;%ld,%ld,%ld,%ld", jup_gz, jupdaldag[0], jupdaldag[1], jupdaldag[2],
  jupdaldag[3], jupdaldag[4], jupmurdag[0], jupmurdag[1], jupmurdag[2], jupmurdag[3],
   jupmurdag[4] );
           if ( full_print )                                  
             fprintf ( fptgt, "%s\n", outbuf );

      sprintf ( outbuf, "Saturn (%ld),  dal-dag: %ld;%ld,%ld,%ld,%ld\
\tmyur-dag: %ld;%ld,%ld,%ld,%ld", sat_gz, satdaldag[0], satdaldag[1], satdaldag[2],
    satdaldag[3], satdaldag[4], satmurdag[0], satmurdag[1], satmurdag[2], satmurdag[3],
    satmurdag[4] );
           if ( full_print )                                  
             fprintf ( fptgt, "%s\n", outbuf );

      sprintf ( outbuf, "Mercury (%ld), dal-dag: %ld;%ld,%ld,%ld,%ld\
\tmyur-dag: %ld;%ld,%ld,%ld,%ld", mer_gz, merdaldag[0], merdaldag[1], merdaldag[2],
merdaldag[3],
    merdaldag[4], mermurdag[0], mermurdag[1], mermurdag[2], mermurdag[3], mermurdag[4] );
           if ( full_print )                                  
             fprintf ( fptgt, "%s\n", outbuf );

      sprintf ( outbuf, "Venus (%ld),   dal-dag: %ld;%ld,%ld,%ld,%ld\
\tmyur-dag: %ld;%ld,%ld,%ld,%ld", ven_gz, vendaldag[0], vendaldag[1], vendaldag[2],
    vendaldag[3], vendaldag[4], venmurdag[0], venmurdag[1], venmurdag[2], venmurdag[3],
    venmurdag[4] );
    if ( full_print )                                  
      fprintf ( fptgt, "%s\n\n", outbuf );

// END OF PLANETS

                 mak_lmchange_string ( cur_monlong, scd_monlong, doweek ); // details calculated above

// LINE 1. - for second duplicated day, only

                sprintf ( outbuf, "%ld: %s. %s(%ld). %s-%s; %s %ld %s %ld",
                          tt, //               XXX
                          dayoweek[ (int) doweek ],
                          lunmanP[ lunmanx ], lunmanx, // XXX
                          fourelem[ dowkelem[ (int) doweek ] ],
                          fourelem[ lunmanelem[ lunmanx ] ],
                          lmchg,
                          wd, wmonths[ (int) (wm-1L) ], wy);
                          
                if ( full_print )                                  
                  fprintf ( fptgt, "%s\n", outbuf );
                fprintf ( fptgt1, "%s\n", outbuf );

// LINE 2.
                sprintf ( outbuf, "%s, %s, %s, %s %ld",
                          yogaP[ sbyorba ],
                          byedP[ byedpax ],
                          cycanim[ ld_animx ],
                          cycpar[ (int) ld_parx ], ld_smex );
                if ( full_print )                                  
                  fprintf ( fptgt, "\x20\x20%s\n", outbuf );
                fprintf ( fptgt1, "\x20\x20%s\n", outbuf );

// LINE 3.
                sprintf ( outbuf,
//              "%ld;%ld,%ld %ld;%ld,%ld %ld;%ld,%ld %ld;%ld,%ld %ld;%ld,%ld",
   "%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld",
                          gzadag[0], gzadag[1], gzadag[2],
                          monlong[0], monlong[1], monlong[2],
                          nyidag[0], nyidag[1], nyidag[2],
                          sbyorlong[0], sbyorlong[1], sbyorlong[2],
                          ZS_s, ZS_d, ZS_m );
                if ( full_print )                                  
                  fprintf ( fptgt, "%s\n", outbuf );
                if ( full_print )                                                    
                  fprintf ( fptgt2, "%s\n", outbuf );

// LINE 4.
                sprintf ( outbuf, "Solar: %s-%s. %s %ld" ,
                          cycelem1[ (int) sd_elemx ],
                          cycanim[ (int) sd_animx ],
                          Clunman[ sd_clunmanx ],
                          sd_smex );
                if ( full_print )                                                            
                  fprintf ( fptgt, "\x20\x20%s\n", outbuf );
                fprintf ( fptgt1, "\x20\x20%s\n", outbuf );

// LINE 5.

                sprintf ( outbuf, "%ld %s %ld",
                          wd, wmonths[ (int) (wm-1L) ], wy);
                if ( full_print )                                                            
                  fprintf ( fptgt, "\x20\x20%s\n", outbuf );
                fprintf ( fptgt1, "\x20\x20%s\n", outbuf );
                }
              else  // NO LHAG - NORMAL DAY
                {
//  CALCULATE LUNAR MANSION AT SUNRISE, AND END OF SOLAR DAY:  ---to here to here to here

                for ( i = 0; i < 6; ++i )
                  {
                    lm1[i] = prv_monlong[i];
                    lm2[i] = monlong[i];
		  }
 		if ( lm1[0] != lm2[0] )
 		  {
 		    if (lm2[0] == 0L )
 		      lm2[0] = 27L;
		  }
		  
                sub_gen ( lmx, lm2, lm1, 27L, sun_f );
                
//                printf ( "DIFFERENCE: " );
//                prn_lst ( lmx );
                
                gzafrac1 = ( prv_gzadag[1] * 60L + prv_gzadag[2] ) * 6L + prv_gzadag[3];
                gzafrac2 = ( gzadag[1] * 60L + gzadag[2] ) * 6L + gzadag[3];
                gzafrac1 = 21600L - gzafrac1;
                mul_gen ( lmx, lmx, gzafrac1, 60000L, sun_f );
                div_g6 ( lmx, ( gzafrac1 + gzafrac2 ), sun_f, 1L );
                add_gen ( cur_monlong, prv_monlong, lmx, 27L, sun_f );
                lunmanx = cur_monlong[0]; 

// NOW, CALCULATE FOR END OF CURRENT, NORMAL, SOLAR DAY:

                for ( i = 0; i < 6; ++i )
                  {
                    lm1[i] = monlong[i];
                    lm2[i] = nxt_monlong[i];
		  }
 		  
 		if ( lm1[0] != lm2[0] )
 		  {
 		    if (lm2[0] == 0L )
 		      lm2[0] = 27L;
		  }
		  
                sub_gen ( lmx, lm2, lm1, 27L, sun_f );
                gzafrac1 = ( gzadag[1] * 60L + gzadag[2] ) * 6L + gzadag[3];
                gzafrac2 = ( nxt_gzadag[1] * 60L + nxt_gzadag[2] ) * 6L + nxt_gzadag[3];
                gzafrac1 = 21600L - gzafrac1;
                mul_gen ( lmx, lmx, gzafrac1, 60000L, sun_f );
                div_g6 ( lmx, ( gzafrac1 + gzafrac2 ), sun_f, 1L );
                add_gen ( scd_monlong, monlong, lmx, 27L, sun_f );
                nxt_lm = scd_monlong[0];

     mak_lmchange_string ( cur_monlong, scd_monlong, doweek );

     get_phrochen ( &phrochen1, &phrochen2, doweek, lunmanx, nxt_lm, monlong[1] );

//  int zaphro, tsephro; // for daily information
     zatse_phro ( &zaphro1, &tsephro1, doweek, tt, lunmanx, monlong[1] );
     zatse_phro ( &zaphro2, &tsephro2, doweek, tt + 1L, nxt_lm, monlong[1] );
     chk_tsephro ( sreg_str, tsephro1, tsephro2, gzadag[1], gzadag[2] );
     byung_phro ( byungphro_str, doweek, lunmanx, monlong[1] );

// CALCULATE SBYORBA:

    add_gen ( lista, cur_monlong, nyidag, 27L, sun_f );
    sbyorba = lista[0];

    for ( i = 0; i < 6; ++i )
      sbyorlong[i] = lista[i];
      
// NOW CALCULATE BYED PA:

    byedpax = calc_byedpa ( cur_monlong, nyidag );

// LINE 1. Normal day - only

      sprintf ( outbuf, "%ld: %s %s %s. %s(%ld), %s-%s, %s, %s %s %s; %s, %s, %s",
                          tt, //           XXXX                         
                          rgyalsogs[(tt-1L)%5L],
                          tseyang[(tt-1L)%5L],
                          dayoweek[ (int) doweek ],
                          lunmanP[ lunmanx ], lunmanx, // XXXX
                          fourelem[ dowkelem[ (int) doweek ] ],
                          fourelem[ lunmanelem[ lunmanx ] ],
                          phrochen[ phrochen1 ],
                          gzaphrod [zaphro1],
                          sreg_str,   // problem with this one???
                          byungphro_str,
                          lmchg,
                          phrochen[ phrochen2 ],
                          gzaphrod[zaphro2] );

//                        wd, wmonths[ (int) (wm-1L) ], wy);

                if ( full_print )                                  
                  fprintf ( fptgt, "%s\n", outbuf );
                fprintf ( fptgt1, "%s\n", outbuf );

// LINE 2.
                sprintf ( outbuf, "%s, %s, %s, %s %ld",
                          yogaP[ sbyorba ], 
                          byedP[ byedpax ],
                          cycanim[ ld_animx ],
                          cycpar[ (int) ld_parx ], ld_smex );
                if ( full_print )                                  
                  fprintf ( fptgt, "\x20\x20%s\n", outbuf );
                fprintf ( fptgt1, "\x20\x20%s\n", outbuf );

// LINE 3.

                sprintf ( outbuf,
//              "%ld;%ld,%ld %ld;%ld,%ld %ld;%ld,%ld %ld;%ld,%ld %ld;%ld,%ld",
   "%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld\n%ld",
                          gzadag[0], gzadag[1], gzadag[2],
                          monlong[0], monlong[1], monlong[2],
                          nyidag[0], nyidag[1], nyidag[2],
                          sbyorlong[0], sbyorlong[1], sbyorlong[2],
                          ZS_s, ZS_d, ZS_m );
                if ( full_print )                                                            
                  fprintf ( fptgt, "%s\n", outbuf );
                if ( full_print )                                                    
                  fprintf ( fptgt2, "%s\n", outbuf );

// LINE 4.
                    sprintf ( outbuf, "Solar: %s-%s. %s %ld" ,
                              cycelem1[ (int) sd_elemx ],
                              cycanim[ (int) sd_animx ],
                              Clunman[ sd_clunmanx ],
                              sd_smex );
                if ( full_print )                                                                
                  fprintf ( fptgt, "\x20\x20%s\n", outbuf );
                fprintf ( fptgt1, "\x20\x20%s\n", outbuf );

// Have extra line here if there is a solar term this day:

           if ( chk_solar_term ( gzadag, nyibar, solar_term_str ))
             {
                fprintf ( fptgt1, "\x20\x20%s\n", solar_term_str );
             }
             
// LINE 5.
                sprintf ( outbuf, "%ld %s %ld",
                          wd, wmonths[ (int) (wm-1L) ], wy);

                if ( full_print )                                  
                  fprintf ( fptgt, "\x20\x20%s\n", outbuf );
                fprintf ( fptgt1, "\x20\x20%s\n", outbuf );

                }

                if ( !delaymth && chk_spec ( mthnum, tt ) )
                  {
//                  printf ( "  %s\n", outbuf );
                    if ( full_print )                                  
                      fprintf ( fptgt, "  %s\n", outbuf );
                    fprintf ( fptgt1, "  %s\n", outbuf );
                  }
              }

// PLANET DATA SHOULD GO HERE:

         if ( !chad)
           {
            if ( lhag )
              ++bir_spizag;
            do_plans ( bir_spizag ); // In T3.C

            sprintf ( outbuf, "Planets (%ld):", bir_spizag );
            if ( full_print )                                  
              fprintf ( fptgt, "\n%s\n\n", outbuf );

      sprintf ( outbuf, "Mars (%ld),    dal-dag: %ld;%ld,%ld,%ld,%ld\
\tmyur-dag: %ld;%ld,%ld,%ld,%ld", mar_gz, mardaldag[0], mardaldag[1], mardaldag[2],
  mardaldag[3], mardaldag[4], marmurdag[0], marmurdag[1], marmurdag[2], marmurdag[3],
   marmurdag[4] );
           if ( full_print )                                  
             fprintf ( fptgt, "%s\n", outbuf );

      sprintf ( outbuf, "Jupiter (%ld), dal-dag: %ld;%ld,%ld,%ld,%ld\
\tmyur-dag: %ld;%ld,%ld,%ld,%ld", jup_gz, jupdaldag[0], jupdaldag[1], jupdaldag[2],
  jupdaldag[3], jupdaldag[4], jupmurdag[0], jupmurdag[1], jupmurdag[2], jupmurdag[3],
   jupmurdag[4] );
           if ( full_print )                                  
             fprintf ( fptgt, "%s\n", outbuf );

      sprintf ( outbuf, "Saturn (%ld),  dal-dag: %ld;%ld,%ld,%ld,%ld\
\tmyur-dag: %ld;%ld,%ld,%ld,%ld", sat_gz, satdaldag[0], satdaldag[1], satdaldag[2],
     satdaldag[3], satdaldag[4], satmurdag[0], satmurdag[1], satmurdag[2], satmurdag[3],
     satmurdag[4] );
           if ( full_print )                                  
             fprintf ( fptgt, "%s\n", outbuf );

      sprintf ( outbuf, "Mercury (%ld), dal-dag: %ld;%ld,%ld,%ld,%ld\
\tmyur-dag: %ld;%ld,%ld,%ld,%ld", mer_gz, merdaldag[0], merdaldag[1], merdaldag[2],
    merdaldag[3], merdaldag[4], mermurdag[0], mermurdag[1], mermurdag[2], mermurdag[3],
    mermurdag[4] );
           if ( full_print )                                      
             fprintf ( fptgt, "%s\n", outbuf );

      sprintf ( outbuf, "Venus (%ld),   dal-dag: %ld;%ld,%ld,%ld,%ld\
\tmyur-dag: %ld;%ld,%ld,%ld,%ld", ven_gz, vendaldag[0], vendaldag[1], vendaldag[2],
    vendaldag[3], vendaldag[4], venmurdag[0], venmurdag[1], venmurdag[2], venmurdag[3],
    venmurdag[4] );
           if ( full_print )                                      
             fprintf ( fptgt, "%s\n\n", outbuf );

           }

// END OF PLANETS

            ++ltt;
            newmth = 0;
            lastjd = juldat;
          } while ( ltt <= 30 ); // END OF DAY LOOP

// END OF DAY LOOP - END OF DAY LOOP - END OF DAY LOOP - END OF DAY LOOP -

        newmth = 1;
        if ( !zeromthfg )
          ++tm;
        if ( tm > 12 )
          {
            ++ty;
            tm = 1;
          }
        if ( adj_mth == 12L )
          more = 0; // We have finished.
      } while ( more );        // END OF MONTH LOOP

    if ( full_print )   
      {                               
        fclose ( fptgt );
        fclose ( fptgt2 );
      }
    fclose ( fptgt1 );  

    if ( cur_year < last_year )
      {
        ++cur_year;
        goto birdonextyear;
      }
  } // END - prn_bir2_cal ()
  
